@@ -1,18 +1,18 @@
-// This is UFC (Unified Form-assembly Code) v. 1.4.
+// This is UFC (Unified Form-assembly Code) v. 2.0.
 // This code is released into the public domain.
 //
-// The FEniCS Project (http://www.fenics.org/) 2006-2010.
+// The FEniCS Project (http://www.fenicsproject.org/) 2006-2011.
 
 #ifndef __UFC_H
 #define __UFC_H
 
-#define UFC_VERSION_MAJOR 1
-#define UFC_VERSION_MINOR 4
-#define UFC_VERSION_MAINTENANCE 1
+#define UFC_VERSION_MAJOR 2
+#define UFC_VERSION_MINOR 0
+#define UFC_VERSION_MAINTENANCE 0
 
 #include <stdexcept>
 
-const char UFC_VERSION[] = "1.4";
+const char UFC_VERSION[] = "2.0";
 
 namespace ufc
 {
@@ -52,7 +52,8 @@ namespace ufc
     /// Constructor
     cell(): cell_shape(interval),
             topological_dimension(0), geometric_dimension(0),
-            entity_indices(0), coordinates(0) {}
+            entity_indices(0), coordinates(0), index(0), local_facet(-1),
+            mesh_identifier(-1) {}
 
     /// Destructor
     virtual ~cell() {}
@@ -72,6 +73,15 @@ namespace ufc
     /// Array of coordinates for the vertices of the cell
     double** coordinates;
 
+    /// Cell index (short-cut for entity_indices[topological_dimension][0])
+    unsigned int index;
+
+    /// Local facet index
+    int local_facet;
+
+    /// Unique mesh identifier
+    int mesh_identifier;
+
   };
 
   /// This class defines the interface for a general tensor-valued function.
@@ -111,6 +121,12 @@ namespace ufc
     /// Return the cell shape
     virtual shape cell_shape() const = 0;
 
+    // Return the topological dimension of the cell shape
+    virtual unsigned int topological_dimension() const = 0;
+
+    // Return the geometric dimension of the cell shape
+    virtual unsigned int geometric_dimension() const = 0;
+
     /// Return the dimension of the finite element function space
     virtual unsigned int space_dimension() const = 0;
 
@@ -159,43 +175,62 @@ namespace ufc
                                            const double* dof_values,
                                            const cell& c) const = 0;
 
+    // Map coordinate xhat from reference cell to coordinate x in cell
+    virtual void map_from_reference_cell(double* x,
+                                         const double* xhat,
+                                         const cell& c) = 0;
+
+    // Map from coordinate x in cell to coordinate xhat in reference cell
+    virtual void map_to_reference_cell(double* xhat,
+                                       const double* x,
+                                       const cell& c) = 0;
+
     /// Return the number of sub elements (for a mixed element)
     virtual unsigned int num_sub_elements() const = 0;
 
     /// Create a new finite element for sub element i (for a mixed element)
     virtual finite_element* create_sub_element(unsigned int i) const = 0;
 
+    /// Create a new class instance
+    virtual finite_element* create() const = 0;
+
   };
 
   /// This class defines the interface for a local-to-global mapping of
   /// degrees of freedom (dofs).
 
-  class dof_map
+  class dofmap
   {
   public:
 
     /// Constructor
-    dof_map() {}
+    dofmap() {}
 
     /// Destructor
-    virtual ~dof_map() {}
+    virtual ~dofmap() {}
 
-    /// Return a string identifying the dof map
+    /// Return a string identifying the dofmap
     virtual const char* signature() const = 0;
 
     /// Return true iff mesh entities of topological dimension d are needed
     virtual bool needs_mesh_entities(unsigned int d) const = 0;
 
-    /// Initialize dof map for mesh (return true iff init_cell() is needed)
+    /// Initialize dofmap for mesh (return true iff init_cell() is needed)
     virtual bool init_mesh(const mesh& mesh) = 0;
 
-    /// Initialize dof map for given cell
+    /// Initialize dofmap for given cell
     virtual void init_cell(const mesh& m,
                            const cell& c) = 0;
 
-    /// Finish initialization of dof map for cells
+    /// Finish initialization of dofmap for cells
     virtual void init_cell_finalize() = 0;
 
+    // Return the topological dimension of the associated cell shape
+    virtual unsigned int topological_dimension() const = 0;
+
+    // Return the geometric dimension of the associated cell shape
+    virtual unsigned int geometric_dimension() const = 0;
+
     /// Return the dimension of the global finite element function space
     virtual unsigned int global_dimension() const = 0;
 
@@ -205,9 +240,6 @@ namespace ufc
     /// Return the maximum dimension of the local finite element function space
     virtual unsigned int max_local_dimension() const = 0;
 
-    // Return the geometric dimension of the coordinates this dof map provides
-    virtual unsigned int geometric_dimension() const = 0;
-
     /// Return the number of dofs on each cell facet
     virtual unsigned int num_facet_dofs() const = 0;
 
@@ -231,11 +263,14 @@ namespace ufc
     virtual void tabulate_coordinates(double** coordinates,
                                       const cell& c) const = 0;
 
-    /// Return the number of sub dof maps (for a mixed element)
-    virtual unsigned int num_sub_dof_maps() const = 0;
+    /// Return the number of sub dofmaps (for a mixed element)
+    virtual unsigned int num_sub_dofmaps() const = 0;
 
-    /// Create a new dof_map for sub dof map i (for a mixed element)
-    virtual dof_map* create_sub_dof_map(unsigned int i) const = 0;
+    /// Create a new dofmap for sub dofmap i (for a mixed element)
+    virtual dofmap* create_sub_dofmap(unsigned int i) const = 0;
+
+    /// Create a new class instance
+    virtual dofmap* create() const = 0;
 
   };
 
@@ -258,6 +293,15 @@ namespace ufc
                                  const double * const * w,
                                  const cell& c) const = 0;
 
+    /// Tabulate the tensor for the contribution from a local cell
+    /// using the specified reference cell quadrature points/weights
+    virtual void tabulate_tensor(double* A,
+                                 const double * const * w,
+                                 const cell& c,
+                                 unsigned int num_quadrature_points,
+                                 const double * const * quadrature_points,
+                                 const double* quadrature_weights) const = 0;
+
   };
 
   /// This class defines the interface for the tabulation of the
@@ -280,6 +324,15 @@ namespace ufc
                                  const cell& c,
                                  unsigned int facet) const = 0;
 
+    /// Tabulate the tensor for the contribution from a local exterior facet
+    /// using the specified reference cell quadrature points/weights
+    virtual void tabulate_tensor(double* A,
+                                 const double * const * w,
+                                 const cell& c,
+                                 unsigned int num_quadrature_points,
+                                 const double * const * quadrature_points,
+                                 const double* quadrature_weights) const = 0;
+
   };
 
   /// This class defines the interface for the tabulation of the
@@ -304,6 +357,15 @@ namespace ufc
                                  unsigned int facet0,
                                  unsigned int facet1) const = 0;
 
+    /// Tabulate the tensor for the contribution from a local interior facet
+    /// using the specified reference cell quadrature points/weights
+    virtual void tabulate_tensor(double* A,
+                                 const double * const * w,
+                                 const cell& c,
+                                 unsigned int num_quadrature_points,
+                                 const double * const * quadrature_points,
+                                 const double* quadrature_weights) const = 0;
+
   };
 
   /// This class defines the interface for the assembly of the global
@@ -340,20 +402,20 @@ namespace ufc
     /// Return the number of coefficients (n)
     virtual unsigned int num_coefficients() const = 0;
 
-    /// Return the number of cell integrals
-    virtual unsigned int num_cell_integrals() const = 0;
+    /// Return the number of cell domains
+    virtual unsigned int num_cell_domains() const = 0;
 
-    /// Return the number of exterior facet integrals
-    virtual unsigned int num_exterior_facet_integrals() const = 0;
+    /// Return the number of exterior facet domains
+    virtual unsigned int num_exterior_facet_domains() const = 0;
 
-    /// Return the number of interior facet integrals
-    virtual unsigned int num_interior_facet_integrals() const = 0;
+    /// Return the number of interior facet domains
+    virtual unsigned int num_interior_facet_domains() const = 0;
 
     /// Create a new finite element for argument function i
     virtual finite_element* create_finite_element(unsigned int i) const = 0;
 
-    /// Create a new dof map for argument function i
-    virtual dof_map* create_dof_map(unsigned int i) const = 0;
+    /// Create a new dofmap for argument function i
+    virtual dofmap* create_dofmap(unsigned int i) const = 0;
 
     /// Create a new cell integral on sub domain i
     virtual cell_integral* create_cell_integral(unsigned int i) const = 0;
